"""
The analyze module is used to analyze backtest results.

It can read csv and json files into memory, calculate various trade performance metrics
and output visualization of the results.
"""

from .analyze import EventAnalyzer, AlphaAnalyzer


__all__ = ['EventAnalyzer', 'AlphaAnalyzer']