/*!
    Blueprint CSS v2.4.0
    http://coryrylan.github.io/blueprint-css/
    License MIT 2017
*/
html {
  box-sizing: border-box; }

*, *::before, *::after {
  box-sizing: inherit; }

[bp-layout~='container'] {
  width: 100%;
  margin: 0 auto;
  display: block;
  max-width: 1000px; }

[bp-layout^='col'] {
  -webkit-flex: 1;
  -ms-flex: 1;
  flex: 1;
  width: 100%;
  padding: 0 8px 16px 8px;
  display: block; }

[bp-layout~='row'] {
  -webkit-flex-wrap: wrap;
  -ms-flex-wrap: wrap;
  flex-wrap: wrap;
  display: -webkit-flex;
  display: -ms-flexbox;
  display: flex;
  margin-left: -8px;
  margin-right: -8px; }

[bp-layout~='middle'] {
  -webkit-align-items: center;
  -ms-flex-align: center;
  align-items: center; }

[bp-layout~='start'] {
  -webkit-align-items: flex-start;
  -ms-flex-align: start;
  align-items: flex-start; }

[bp-layout~='end'] {
  -webkit-align-items: flex-end;
  -ms-flex-align: end;
  align-items: flex-end; }

[bp-layout~='center'] {
  -webkit-justify-content: center;
  -ms-flex-pack: center;
  justify-content: center; }

[bp-layout~='reverse'] {
  -webkit-flex-direction: row-reverse;
  -ms-flex-direction: row-reverse;
  flex-direction: row-reverse; }

[bp-layout~='around'] {
  -webkit-justify-content: space-around;
  -ms-flex-pack: distribute;
  justify-content: space-around; }

[bp-layout~='between'] {
  -webkit-justify-content: space-between;
  -ms-flex-pack: justify;
  justify-content: space-between; }

[bp-layout~='stretch'] {
  align-items: stretch;
  flex-flow: row wrap; }

[bp-layout~='stretch'] > [bp-layout~='col'] {
  display: -webkit-flex;
  display: -ms-flexbox;
  display: flex; }

[bp-layout~='gutter-none'] > [bp-layout~='col'] {
  padding: 0; }

[bp-layout~='gutter-bottom-none'] > [bp-layout~='col'] {
  padding: 0 8px 0 8px; }

[bp-layout~='gutter-between-none'] > [bp-layout~='col'] {
  padding: 0 0 16px 0; }

[bp-layout~='gutter-none'] > [bp-layout~='col'] {
  padding: 0; }

[bp-layout~='gutter-none'], [bp-layout~='gutter-between-none'] {
  margin-left: 0 !important;
  margin-right: 0 !important; }

[bp-layout~='first'] {
  -webkit-order: -1;
  -ms-flex-order: -1;
  order: -1; }

[bp-layout~='last'] {
  -webkit-order: 1;
  -ms-flex-order: 1;
  order: 1; }

[bp-layout~='hide'] {
  display: none !important; }

[bp-layout~='show'] {
  display: block !important; }

[bp-layout~='1'] {
  max-width: 8.33333%;
  width: 8.33333%;
  -webkit-flex: 8.33333%;
  -ms-flex: 8.33333%;
  flex: 8.33333%; }

[bp-layout~='1@sm'], [bp-layout~='1@md'], [bp-layout~='1@lg'], [bp-layout~='1@xl'] {
  -webkit-flex: 0 0 auto;
  -ms-flex: 0 0 auto;
  flex: 0 0 auto; }

[bp-layout~='2'] {
  max-width: 16.66667%;
  width: 16.66667%;
  -webkit-flex: 16.66667%;
  -ms-flex: 16.66667%;
  flex: 16.66667%; }

[bp-layout~='2@sm'], [bp-layout~='2@md'], [bp-layout~='2@lg'], [bp-layout~='2@xl'] {
  -webkit-flex: 0 0 auto;
  -ms-flex: 0 0 auto;
  flex: 0 0 auto; }

[bp-layout~='3'] {
  max-width: 25%;
  width: 25%;
  -webkit-flex: 25%;
  -ms-flex: 25%;
  flex: 25%; }

[bp-layout~='3@sm'], [bp-layout~='3@md'], [bp-layout~='3@lg'], [bp-layout~='3@xl'] {
  -webkit-flex: 0 0 auto;
  -ms-flex: 0 0 auto;
  flex: 0 0 auto; }

[bp-layout~='4'] {
  max-width: 33.33333%;
  width: 33.33333%;
  -webkit-flex: 33.33333%;
  -ms-flex: 33.33333%;
  flex: 33.33333%; }

[bp-layout~='4@sm'], [bp-layout~='4@md'], [bp-layout~='4@lg'], [bp-layout~='4@xl'] {
  -webkit-flex: 0 0 auto;
  -ms-flex: 0 0 auto;
  flex: 0 0 auto; }

[bp-layout~='5'] {
  max-width: 41.66667%;
  width: 41.66667%;
  -webkit-flex: 41.66667%;
  -ms-flex: 41.66667%;
  flex: 41.66667%; }

[bp-layout~='5@sm'], [bp-layout~='5@md'], [bp-layout~='5@lg'], [bp-layout~='5@xl'] {
  -webkit-flex: 0 0 auto;
  -ms-flex: 0 0 auto;
  flex: 0 0 auto; }

[bp-layout~='6'] {
  max-width: 50%;
  width: 50%;
  -webkit-flex: 50%;
  -ms-flex: 50%;
  flex: 50%; }

[bp-layout~='6@sm'], [bp-layout~='6@md'], [bp-layout~='6@lg'], [bp-layout~='6@xl'] {
  -webkit-flex: 0 0 auto;
  -ms-flex: 0 0 auto;
  flex: 0 0 auto; }

[bp-layout~='7'] {
  max-width: 58.33333%;
  width: 58.33333%;
  -webkit-flex: 58.33333%;
  -ms-flex: 58.33333%;
  flex: 58.33333%; }

[bp-layout~='7@sm'], [bp-layout~='7@md'], [bp-layout~='7@lg'], [bp-layout~='7@xl'] {
  -webkit-flex: 0 0 auto;
  -ms-flex: 0 0 auto;
  flex: 0 0 auto; }

[bp-layout~='8'] {
  max-width: 66.66667%;
  width: 66.66667%;
  -webkit-flex: 66.66667%;
  -ms-flex: 66.66667%;
  flex: 66.66667%; }

[bp-layout~='8@sm'], [bp-layout~='8@md'], [bp-layout~='8@lg'], [bp-layout~='8@xl'] {
  -webkit-flex: 0 0 auto;
  -ms-flex: 0 0 auto;
  flex: 0 0 auto; }

[bp-layout~='9'] {
  max-width: 75%;
  width: 75%;
  -webkit-flex: 75%;
  -ms-flex: 75%;
  flex: 75%; }

[bp-layout~='9@sm'], [bp-layout~='9@md'], [bp-layout~='9@lg'], [bp-layout~='9@xl'] {
  -webkit-flex: 0 0 auto;
  -ms-flex: 0 0 auto;
  flex: 0 0 auto; }

[bp-layout~='10'] {
  max-width: 83.33333%;
  width: 83.33333%;
  -webkit-flex: 83.33333%;
  -ms-flex: 83.33333%;
  flex: 83.33333%; }

[bp-layout~='10@sm'], [bp-layout~='10@md'], [bp-layout~='10@lg'], [bp-layout~='10@xl'] {
  -webkit-flex: 0 0 auto;
  -ms-flex: 0 0 auto;
  flex: 0 0 auto; }

[bp-layout~='11'] {
  max-width: 91.66667%;
  width: 91.66667%;
  -webkit-flex: 91.66667%;
  -ms-flex: 91.66667%;
  flex: 91.66667%; }

[bp-layout~='11@sm'], [bp-layout~='11@md'], [bp-layout~='11@lg'], [bp-layout~='11@xl'] {
  -webkit-flex: 0 0 auto;
  -ms-flex: 0 0 auto;
  flex: 0 0 auto; }

[bp-layout~='12'] {
  max-width: 100%;
  width: 100%;
  -webkit-flex: 100%;
  -ms-flex: 100%;
  flex: 100%; }

[bp-layout~='12@sm'], [bp-layout~='12@md'], [bp-layout~='12@lg'], [bp-layout~='12@xl'] {
  -webkit-flex: 0 0 auto;
  -ms-flex: 0 0 auto;
  flex: 0 0 auto; }

[bp-layout~='12'] {
  -webkit-flex: 0 0 auto;
  -ms-flex: 0 0 auto;
  flex: 0 0 auto; }

[bp-layout~='offset-1'] {
  margin-left: 8.33333%; }

[bp-layout~='offset-2'] {
  margin-left: 16.66667%; }

[bp-layout~='offset-3'] {
  margin-left: 25%; }

[bp-layout~='offset-4'] {
  margin-left: 33.33333%; }

[bp-layout~='offset-5'] {
  margin-left: 41.66667%; }

[bp-layout~='offset-6'] {
  margin-left: 50%; }

[bp-layout~='offset-7'] {
  margin-left: 58.33333%; }

[bp-layout~='offset-8'] {
  margin-left: 66.66667%; }

[bp-layout~='offset-9'] {
  margin-left: 75%; }

[bp-layout~='offset-10'] {
  margin-left: 83.33333%; }

[bp-layout~='offset-11'] {
  margin-left: 91.66667%; }

[bp-layout~='offset-12'] {
  margin-left: 100%; }

@media (min-width: 30em) {
  [bp-layout~='1@sm'] {
    max-width: 8.33333%;
    width: 8.33333%; }
  [bp-layout~='2@sm'] {
    max-width: 16.66667%;
    width: 16.66667%; }
  [bp-layout~='3@sm'] {
    max-width: 25%;
    width: 25%; }
  [bp-layout~='4@sm'] {
    max-width: 33.33333%;
    width: 33.33333%; }
  [bp-layout~='5@sm'] {
    max-width: 41.66667%;
    width: 41.66667%; }
  [bp-layout~='6@sm'] {
    max-width: 50%;
    width: 50%; }
  [bp-layout~='7@sm'] {
    max-width: 58.33333%;
    width: 58.33333%; }
  [bp-layout~='8@sm'] {
    max-width: 66.66667%;
    width: 66.66667%; }
  [bp-layout~='9@sm'] {
    max-width: 75%;
    width: 75%; }
  [bp-layout~='10@sm'] {
    max-width: 83.33333%;
    width: 83.33333%; }
  [bp-layout~='11@sm'] {
    max-width: 91.66667%;
    width: 91.66667%; }
  [bp-layout~='12@sm'] {
    max-width: 100%;
    width: 100%; }
  [bp-layout~='offset-1@sm'] {
    margin-left: 8.33333%; }
  [bp-layout~='offset-2@sm'] {
    margin-left: 16.66667%; }
  [bp-layout~='offset-3@sm'] {
    margin-left: 25%; }
  [bp-layout~='offset-4@sm'] {
    margin-left: 33.33333%; }
  [bp-layout~='offset-5@sm'] {
    margin-left: 41.66667%; }
  [bp-layout~='offset-6@sm'] {
    margin-left: 50%; }
  [bp-layout~='offset-7@sm'] {
    margin-left: 58.33333%; }
  [bp-layout~='offset-8@sm'] {
    margin-left: 66.66667%; }
  [bp-layout~='offset-9@sm'] {
    margin-left: 75%; }
  [bp-layout~='offset-10@sm'] {
    margin-left: 83.33333%; }
  [bp-layout~='offset-11@sm'] {
    margin-left: 91.66667%; }
  [bp-layout~='offset-12@sm'] {
    margin-left: 100%; }
  [bp-layout~='hide@sm'] {
    display: none !important; }
  [bp-layout~='show@sm'] {
    display: block !important; }
  [bp-layout~='first@sm'] {
    -webkit-order: -1;
    -ms-flex-order: -1;
    order: -1; }
  [bp-layout~='last@sm'] {
    -webkit-order: 1;
    -ms-flex-order: 1;
    order: 1; } }

@media (min-width: 45em) {
  [bp-layout~='1@md'] {
    max-width: 8.33333%;
    width: 8.33333%; }
  [bp-layout~='2@md'] {
    max-width: 16.66667%;
    width: 16.66667%; }
  [bp-layout~='3@md'] {
    max-width: 25%;
    width: 25%; }
  [bp-layout~='4@md'] {
    max-width: 33.33333%;
    width: 33.33333%; }
  [bp-layout~='5@md'] {
    max-width: 41.66667%;
    width: 41.66667%; }
  [bp-layout~='6@md'] {
    max-width: 50%;
    width: 50%; }
  [bp-layout~='7@md'] {
    max-width: 58.33333%;
    width: 58.33333%; }
  [bp-layout~='8@md'] {
    max-width: 66.66667%;
    width: 66.66667%; }
  [bp-layout~='9@md'] {
    max-width: 75%;
    width: 75%; }
  [bp-layout~='10@md'] {
    max-width: 83.33333%;
    width: 83.33333%; }
  [bp-layout~='11@md'] {
    max-width: 91.66667%;
    width: 91.66667%; }
  [bp-layout~='12@md'] {
    max-width: 100%;
    width: 100%; }
  [bp-layout~='offset-1@md'] {
    margin-left: 8.33333%; }
  [bp-layout~='offset-2@md'] {
    margin-left: 16.66667%; }
  [bp-layout~='offset-3@md'] {
    margin-left: 25%; }
  [bp-layout~='offset-4@md'] {
    margin-left: 33.33333%; }
  [bp-layout~='offset-5@md'] {
    margin-left: 41.66667%; }
  [bp-layout~='offset-6@md'] {
    margin-left: 50%; }
  [bp-layout~='offset-7@md'] {
    margin-left: 58.33333%; }
  [bp-layout~='offset-8@md'] {
    margin-left: 66.66667%; }
  [bp-layout~='offset-9@md'] {
    margin-left: 75%; }
  [bp-layout~='offset-10@md'] {
    margin-left: 83.33333%; }
  [bp-layout~='offset-11@md'] {
    margin-left: 91.66667%; }
  [bp-layout~='offset-12@md'] {
    margin-left: 100%; }
  [bp-layout~='hide@md'] {
    display: none !important; }
  [bp-layout~='show@md'] {
    display: block !important; }
  [bp-layout~='first@md'] {
    -webkit-order: -1;
    -ms-flex-order: -1;
    order: -1; }
  [bp-layout~='last@md'] {
    -webkit-order: 1;
    -ms-flex-order: 1;
    order: 1; } }

@media (min-width: 60em) {
  [bp-layout~='1@lg'] {
    max-width: 8.33333%;
    width: 8.33333%; }
  [bp-layout~='2@lg'] {
    max-width: 16.66667%;
    width: 16.66667%; }
  [bp-layout~='3@lg'] {
    max-width: 25%;
    width: 25%; }
  [bp-layout~='4@lg'] {
    max-width: 33.33333%;
    width: 33.33333%; }
  [bp-layout~='5@lg'] {
    max-width: 41.66667%;
    width: 41.66667%; }
  [bp-layout~='6@lg'] {
    max-width: 50%;
    width: 50%; }
  [bp-layout~='7@lg'] {
    max-width: 58.33333%;
    width: 58.33333%; }
  [bp-layout~='8@lg'] {
    max-width: 66.66667%;
    width: 66.66667%; }
  [bp-layout~='9@lg'] {
    max-width: 75%;
    width: 75%; }
  [bp-layout~='10@lg'] {
    max-width: 83.33333%;
    width: 83.33333%; }
  [bp-layout~='11@lg'] {
    max-width: 91.66667%;
    width: 91.66667%; }
  [bp-layout~='12@lg'] {
    max-width: 100%;
    width: 100%; }
  [bp-layout~='offset-1@lg'] {
    margin-left: 8.33333%; }
  [bp-layout~='offset-2@lg'] {
    margin-left: 16.66667%; }
  [bp-layout~='offset-3@lg'] {
    margin-left: 25%; }
  [bp-layout~='offset-4@lg'] {
    margin-left: 33.33333%; }
  [bp-layout~='offset-5@lg'] {
    margin-left: 41.66667%; }
  [bp-layout~='offset-6@lg'] {
    margin-left: 50%; }
  [bp-layout~='offset-7@lg'] {
    margin-left: 58.33333%; }
  [bp-layout~='offset-8@lg'] {
    margin-left: 66.66667%; }
  [bp-layout~='offset-9@lg'] {
    margin-left: 75%; }
  [bp-layout~='offset-10@lg'] {
    margin-left: 83.33333%; }
  [bp-layout~='offset-11@lg'] {
    margin-left: 91.66667%; }
  [bp-layout~='offset-12@lg'] {
    margin-left: 100%; }
  [bp-layout~='hide@lg'] {
    display: none !important; }
  [bp-layout~='show@lg'] {
    display: block !important; }
  [bp-layout~='first@lg'] {
    -webkit-order: -1;
    -ms-flex-order: -1;
    order: -1; }
  [bp-layout~='last@lg'] {
    -webkit-order: 1;
    -ms-flex-order: 1;
    order: 1; } }

@media (min-width: 90em) {
  [bp-layout~='1@xl'] {
    max-width: 8.33333%;
    width: 8.33333%; }
  [bp-layout~='2@xl'] {
    max-width: 16.66667%;
    width: 16.66667%; }
  [bp-layout~='3@xl'] {
    max-width: 25%;
    width: 25%; }
  [bp-layout~='4@xl'] {
    max-width: 33.33333%;
    width: 33.33333%; }
  [bp-layout~='5@xl'] {
    max-width: 41.66667%;
    width: 41.66667%; }
  [bp-layout~='6@xl'] {
    max-width: 50%;
    width: 50%; }
  [bp-layout~='7@xl'] {
    max-width: 58.33333%;
    width: 58.33333%; }
  [bp-layout~='8@xl'] {
    max-width: 66.66667%;
    width: 66.66667%; }
  [bp-layout~='9@xl'] {
    max-width: 75%;
    width: 75%; }
  [bp-layout~='10@xl'] {
    max-width: 83.33333%;
    width: 83.33333%; }
  [bp-layout~='11@xl'] {
    max-width: 91.66667%;
    width: 91.66667%; }
  [bp-layout~='12@xl'] {
    max-width: 100%;
    width: 100%; }
  [bp-layout~='offset-1@xl'] {
    margin-left: 8.33333%; }
  [bp-layout~='offset-2@xl'] {
    margin-left: 16.66667%; }
  [bp-layout~='offset-3@xl'] {
    margin-left: 25%; }
  [bp-layout~='offset-4@xl'] {
    margin-left: 33.33333%; }
  [bp-layout~='offset-5@xl'] {
    margin-left: 41.66667%; }
  [bp-layout~='offset-6@xl'] {
    margin-left: 50%; }
  [bp-layout~='offset-7@xl'] {
    margin-left: 58.33333%; }
  [bp-layout~='offset-8@xl'] {
    margin-left: 66.66667%; }
  [bp-layout~='offset-9@xl'] {
    margin-left: 75%; }
  [bp-layout~='offset-10@xl'] {
    margin-left: 83.33333%; }
  [bp-layout~='offset-11@xl'] {
    margin-left: 91.66667%; }
  [bp-layout~='offset-12@xl'] {
    margin-left: 100%; }
  [bp-layout~='hide@xl'] {
    display: none !important; }
  [bp-layout~='show@xl'] {
    display: block !important; }
  [bp-layout~='first@xl'] {
    -webkit-order: -1;
    -ms-flex-order: -1;
    order: -1; }
  [bp-layout~='last@xl'] {
    -webkit-order: 1;
    -ms-flex-order: 1;
    order: 1; } }

[bp-layout~='flex'] {
  -webkit-flex-wrap: wrap;
  -ms-flex-wrap: wrap;
  flex-wrap: wrap;
  -js-display: flex;
  display: -webkit-flex;
  display: -ms-flexbox;
  display: flex; }

[bp-layout~='fill'] {
  -webkit-box-flex: 1;
  -ms-flex: 1 1 0%;
  flex: 1 1 0%;
  -ms-flex-preferred-size: 0%;
  flex-basis: 0%; }

[bp-layout~='fit'] {
  flex-basis: auto; }

[bp-layout~='float-center'] {
  margin-left: auto;
  margin-right: auto;
  display: block;
  float: none; }

[bp-layout~='float-left'] {
  float: left; }

[bp-layout~='float-right'] {
  float: right; }

[bp-layout~='clear-fix']::after {
  content: '';
  display: table;
  clear: both; }

[bp-layout~='full-width'] {
  width: 100%; }

[bp-layout~='text-left'] {
  text-align: left; }

[bp-layout~='text-right'] {
  text-align: right; }

[bp-layout~='text-center'] {
  text-align: center; }

[bp-layout~='1--max'] {
  max-width: 83.33333px !important; }

[bp-layout~='2--max'] {
  max-width: 166.66667px !important; }

[bp-layout~='3--max'] {
  max-width: 250px !important; }

[bp-layout~='4--max'] {
  max-width: 333.33333px !important; }

[bp-layout~='5--max'] {
  max-width: 416.66667px !important; }

[bp-layout~='6--max'] {
  max-width: 500px !important; }

[bp-layout~='7--max'] {
  max-width: 583.33333px !important; }

[bp-layout~='8--max'] {
  max-width: 666.66667px !important; }

[bp-layout~='9--max'] {
  max-width: 750px !important; }

[bp-layout~='10--max'] {
  max-width: 833.33333px !important; }

[bp-layout~='11--max'] {
  max-width: 916.66667px !important; }

[bp-layout~='12--max'] {
  max-width: 1000px !important; }

[bp-layout~='margin--xs'] {
  margin: 4px !important; }

[bp-layout~='margin-top--xs'] {
  margin-top: 4px !important; }

[bp-layout~='margin-bottom--xs'] {
  margin-bottom: 4px !important; }

[bp-layout~='margin-right--xs'] {
  margin-right: 4px !important; }

[bp-layout~='margin-left--xs'] {
  margin-left: 4px !important; }

[bp-layout~='padding--xs'] {
  padding: 4px !important; }

[bp-layout~='padding-top--xs'] {
  padding-top: 4px !important; }

[bp-layout~='padding-bottom--xs'] {
  padding-bottom: 4px !important; }

[bp-layout~='padding-right--xs'] {
  padding-right: 4px !important; }

[bp-layout~='padding-left--xs'] {
  padding-left: 4px !important; }

[bp-layout~='margin--sm'] {
  margin: 8px !important; }

[bp-layout~='margin-top--sm'] {
  margin-top: 8px !important; }

[bp-layout~='margin-bottom--sm'] {
  margin-bottom: 8px !important; }

[bp-layout~='margin-right--sm'] {
  margin-right: 8px !important; }

[bp-layout~='margin-left--sm'] {
  margin-left: 8px !important; }

[bp-layout~='padding--sm'] {
  padding: 8px !important; }

[bp-layout~='padding-top--sm'] {
  padding-top: 8px !important; }

[bp-layout~='padding-bottom--sm'] {
  padding-bottom: 8px !important; }

[bp-layout~='padding-right--sm'] {
  padding-right: 8px !important; }

[bp-layout~='padding-left--sm'] {
  padding-left: 8px !important; }

[bp-layout~='margin'] {
  margin: 16px !important; }

[bp-layout~='margin-top'] {
  margin-top: 16px !important; }

[bp-layout~='margin-bottom'] {
  margin-bottom: 16px !important; }

[bp-layout~='margin-right'] {
  margin-right: 16px !important; }

[bp-layout~='margin-left'] {
  margin-left: 16px !important; }

[bp-layout~='padding'] {
  padding: 16px !important; }

[bp-layout~='padding-top'] {
  padding-top: 16px !important; }

[bp-layout~='padding-bottom'] {
  padding-bottom: 16px !important; }

[bp-layout~='padding-right'] {
  padding-right: 16px !important; }

[bp-layout~='padding-left'] {
  padding-left: 16px !important; }

[bp-layout~='margin--lg'] {
  margin: 24px !important; }

[bp-layout~='margin-top--lg'] {
  margin-top: 24px !important; }

[bp-layout~='margin-bottom--lg'] {
  margin-bottom: 24px !important; }

[bp-layout~='margin-right--lg'] {
  margin-right: 24px !important; }

[bp-layout~='margin-left--lg'] {
  margin-left: 24px !important; }

[bp-layout~='padding--lg'] {
  padding: 24px !important; }

[bp-layout~='padding-top--lg'] {
  padding-top: 24px !important; }

[bp-layout~='padding-bottom--lg'] {
  padding-bottom: 24px !important; }

[bp-layout~='padding-right--lg'] {
  padding-right: 24px !important; }

[bp-layout~='padding-left--lg'] {
  padding-left: 24px !important; }

[bp-layout~='margin--none'] {
  margin: 0 !important; }

[bp-layout~='margin-top--none'] {
  margin-top: 0 !important; }

[bp-layout~='margin-bottom--none'] {
  margin-bottom: 0 !important; }

[bp-layout~='margin-right--none'] {
  margin-right: 0 !important; }

[bp-layout~='margin-left--none'] {
  margin-left: 0 !important; }

[bp-layout~='padding--none'] {
  padding: 0 !important; }

[bp-layout~='padding-top--none'] {
  padding-top: 0 !important; }

[bp-layout~='padding-bottom--none'] {
  padding-bottom: 0 !important; }

[bp-layout~='padding-right--none'] {
  padding-right: 0 !important; }

[bp-layout~='padding-left--none'] {
  padding-left: 0 !important; }

[bp-layout~='col']::after, [bp-layout~='row'][bp-layout~='col']::after {
  content: 'Error: Invalid Grid Structure https://blueprintcss.io#errors';
  background-color: #ec5151;
  padding: 4px 8px;
  color: #fff; }

[bp-layout~='row'] [bp-layout~='col']::after {
  display: none; }
